# Pandas is used for data manipulation
import pandas as pd
# Read in data and display first 5 rows
features = pd.read_csv(r'...\Model_8.csv')
features.head(5)

print('The shape of our features is:', features.shape)

# Descriptive statistics for each column
features.describe()
# Use numpy to convert to arrays
import numpy as np
# Labels are the values we want to predict
labels = np.array(features['ERROR_3D'])
# Remove the labels from the features
# axis 1 refers to the columns
features= features.drop('ERROR_3D', axis = 1)
# Saving feature names for later use
feature_list = list(features.columns)
# Convert to numpy array
features = np.array(features)
# Using Scikit-learn to split data into training and testing sets
from sklearn.model_selection import train_test_split
# Split the data into training and testing sets
train_features, test_features, train_labels, test_labels = train_test_split(features, labels, test_size = 0.25, random_state = 42)

print('Training Features Shape:', train_features.shape)
print('Training Labels Shape:', train_labels.shape)
print('Testing Features Shape:', test_features.shape)
print('Testing Labels Shape:', test_labels.shape)

# # The baseline predictions are the historical averages
# baseline_preds = test_features[:, feature_list.index('average')]
# # Baseline errors, and display average baseline error
# baseline_errors = abs(baseline_preds - test_labels)
# print('Average baseline error: ', round(np.mean(baseline_errors), 2))

# Import the model we are using
from sklearn.ensemble import RandomForestRegressor
# Instantiate model with 1000 decision trees
rf = RandomForestRegressor(n_estimators = 1000, criterion = 'mse', oob_score = True, random_state = 42)
# rf = RandomForestRegressor(n_estimators = 1000, criterion = 'mse', oob_score = True)
# rf = RandomForestRegressor(n_estimators = 1000, criterion = 'mae', random_state = 42)
# Train the model on training data
rf.fit(train_features, train_labels);

# Use the forest's predict method on the test data
predictions = rf.predict(test_features)

# Calculate the absolute errors
errors = abs(predictions - test_labels)
# Print out the mean absolute error (mae)
# print('Mean Absolute Error:', round(np.mean(errors), 6), 'metres')

from sklearn import metrics
print('-----Sklearn metrics-----')
print('Mean Absolute Error:', round(metrics.mean_absolute_error(test_labels, predictions),4))  
print('Mean Squared Error:', round(metrics.mean_squared_error(test_labels, predictions),4))  
print('Root Mean Squared Error:', round(np.sqrt(metrics.mean_squared_error(test_labels, predictions)),4))
from sklearn.metrics import r2_score
print('R2 score:', round(r2_score(test_labels, predictions),4)) # Coefficient of determination. 
print('OOB score:', round((rf.oob_score_),4))

# Calculate mean absolute percentage error (MAPE)
mape = 100 * (errors / test_labels)
# Calculate and display accuracy
accuracy = 100 - np.mean(mape)
print('Accuracy:', round(accuracy, 4), '%.')

# Training accuracy
print(rf.score(train_features, train_labels))
# Test accuracy
print(rf.score(test_features, test_labels))
# Get numerical feature importances
importances = list(rf.feature_importances_)
# List of tuples with variable and importance
feature_importances = [(feature, round(importance, 6)) for feature, importance in zip(feature_list, importances)]
# Sort the feature importances by most important first
feature_importances = sorted(feature_importances, key = lambda x: x[1], reverse = True)
# Print out the feature and importances 
[print('{:20}{}'.format(*pair)) for pair in feature_importances];

# Use these to print out the predictions, test labels and errors
# print(test_labels.tolist())
# print(errors.tolist())
# print(predictions.tolist())